Method: Docker::Compose::Mapper#initialize

Defined in:
lib/docker/compose/mapper.rb

#initialize(session, docker_host, strict: true) ⇒ Mapper

Create an instance of Mapper

Parameters:

  • session (Docker::Compose::Session)
  • docker_host (String)

    DNS hostnrame or IPv4 address of the host that is publishing Docker services (i.e. the DOCKER_HOST hostname or IP if you are using a non-clustered Docker environment)

  • strict (Boolean) (defaults to: true)

    if true, raise BadSubstitution when unrecognized syntax is passed to #map; if false, simply return unrecognized values without substituting anything



42
43
44
45
46
# File 'lib/docker/compose/mapper.rb', line 42

def initialize(session, docker_host, strict:true)
  @session = session
  @docker_host = docker_host
  @strict  = strict
end